pub struct AnimationGraph {
pub nodes: HashMap<NodeId, AnimationNode>,
pub edges: HashMap<SourcePin, TargetPin>,
pub edges_inverted: HashMap<TargetPin, SourcePin>,
pub io_spec: IoSpec<GraphInputPin>,
pub default_data: HashMap<GraphInputPin, DataValue>,
pub editor_metadata: EditorMetadata,
}Fields
nodes: HashMap<NodeId, AnimationNode>edges: HashMap<SourcePin, TargetPin>Indexed by start pin.
edges_inverted: HashMap<TargetPin, SourcePin>Inverted, indexed by end pin.
io_spec: IoSpec<GraphInputPin>Defines inputs and outputs for this graph.
default_data: HashMap<GraphInputPin, DataValue>editor_metadata: EditorMetadataImplementations
Sourceimpl AnimationGraph
impl AnimationGraph
pub fn new() -> AnimationGraph
Sourcepub fn add_node(&mut self, node: AnimationNode)
pub fn add_node(&mut self, node: AnimationNode)
Add a new node to the graph
Sourcepub fn remove_node(&mut self, node_id: impl Into<NodeId>)
pub fn remove_node(&mut self, node_id: impl Into<NodeId>)
Add a new node to the graph
Sourcepub fn add_edge(&mut self, source_pin: SourcePin, target_pin: TargetPin)
pub fn add_edge(&mut self, source_pin: SourcePin, target_pin: TargetPin)
Add a new edge to the graph
Sourcepub fn remove_edge_by_target(
&mut self,
target_pin: &TargetPin,
) -> Option<SourcePin>
pub fn remove_edge_by_target( &mut self, target_pin: &TargetPin, ) -> Option<SourcePin>
Remove an edge from the graph.
Sourcepub fn set_default_data(&mut self, input: GraphInputPin, value: DataValue)
pub fn set_default_data(&mut self, input: GraphInputPin, value: DataValue)
Sets the value for a default parameter, registering it if it wasn’t yet done
Sourcepub fn get_default_data(&mut self, input: &GraphInputPin) -> Option<DataValue>
pub fn get_default_data(&mut self, input: &GraphInputPin) -> Option<DataValue>
Get the default value of an input parameter, if it exists
Sourcepub fn add_input_data(&mut self, input: GraphInputPin, data_spec: DataSpec)
pub fn add_input_data(&mut self, input: GraphInputPin, data_spec: DataSpec)
Register an input pose pin for the graph
Sourcepub fn add_input_time(&mut self, input: GraphInputPin)
pub fn add_input_time(&mut self, input: GraphInputPin)
Register an input pose pin for the graph
Sourcepub fn add_output_data(&mut self, pin_id: String, spec: DataSpec)
pub fn add_output_data(&mut self, pin_id: String, spec: DataSpec)
Register an output parameter for the graph
Sourcepub fn add_output_time(&mut self)
pub fn add_output_time(&mut self)
Enables time “output” for this graph
pub fn add_input_data_edge( &mut self, input_name: impl Into<GraphInputPin>, target_node: impl Into<NodeId>, target_edge: impl Into<String>, )
pub fn add_output_data_edge( &mut self, source_node: impl Into<NodeId>, source_edge: impl Into<String>, output_name: impl Into<String>, )
pub fn add_input_time_edge( &mut self, input_name: impl Into<GraphInputPin>, target_node: impl Into<NodeId>, target_edge: impl Into<String>, )
pub fn add_output_pose_edge(&mut self, source_node: impl Into<NodeId>)
Sourcepub fn add_node_parameter_edge(
&mut self,
source_node: impl Into<NodeId>,
source_pin: impl Into<String>,
target_node: impl Into<NodeId>,
target_pin: impl Into<String>,
)
pub fn add_node_parameter_edge( &mut self, source_node: impl Into<NodeId>, source_pin: impl Into<String>, target_node: impl Into<NodeId>, target_pin: impl Into<String>, )
Adds an edge between two nodes in the graph
Sourcepub fn add_node_pose_edge(
&mut self,
source_node: impl Into<NodeId>,
target_node: impl Into<NodeId>,
target_pin: impl Into<String>,
)
pub fn add_node_pose_edge( &mut self, source_node: impl Into<NodeId>, target_node: impl Into<NodeId>, target_pin: impl Into<String>, )
Adds an edge between two node pose pins in the graph
pub fn validate(&self) -> Result<(), GraphValidationError>
Sourcepub fn can_add_edge(
&self,
edge: Edge,
ctx: SpecResources<'_>,
) -> Result<(), Option<Edge>>
pub fn can_add_edge( &self, edge: Edge, ctx: SpecResources<'_>, ) -> Result<(), Option<Edge>>
Check whether a new edge can be added to the graph. If not, return whether an edge can be removed to maybe make it possible. It is not guaranteed that the edge will be legal after a single edge removal, so this function should be called repeatedly until it returns Ok(()) or Err(None)
Sourcepub fn validate_edges(
&self,
ctx: SpecResources<'_>,
) -> Result<(), HashSet<Edge>>
pub fn validate_edges( &self, ctx: SpecResources<'_>, ) -> Result<(), HashSet<Edge>>
Verify that graph edges are legal. If not, return a set of edges that when removed would make the graph legal.
Reasons for edges to make the graph illegal are:
- Two pose edges share the same source.
- An edge source and target pins have different types. This could be:
- Pose pin connected to a parameter pin.
- Pose type mismatch.
- Parameter type mismatch.
- An edge source pin, target pin or both are missing. This could be because:
- The source node, target node or both are missing.
- The source node or target node do not have the named pin.
- Cycle.
pub fn contains_node_that<T>(&self, f: impl Fn(&T) -> bool) -> Option<NodeId>where
T: NodeLike,
pub fn get_data( &self, target_pin: TargetPin, ctx: GraphContext<'_>, ) -> Result<DataValue, GraphError>
pub fn get_duration( &self, target_pin: TargetPin, ctx: GraphContext<'_>, ) -> Result<Option<f32>, GraphError>
pub fn get_time_update( &self, source_pin: SourcePin, ctx: GraphContext<'_>, ) -> Result<TimeUpdate, GraphError>
pub fn query( &self, time_update: TimeUpdate, context_arena: &mut GraphContextArena, resources: &SystemResources<'_, '_>, root_entity: Entity, entity_map: &HashMap<BoneId, Entity>, deferred_gizmos: &mut DeferredGizmos, ) -> Result<HashMap<String, DataValue>, GraphError>
pub fn query_with_env( &self, time_update: TimeUpdate, context_arena: &mut GraphContextArena, resources: &SystemResources<'_, '_>, io_env: &dyn GraphIoEnv, root_entity: Entity, entity_map: &HashMap<BoneId, Entity>, deferred_gizmos: &mut DeferredGizmos, ) -> Result<HashMap<String, DataValue>, GraphError>